"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const schematics_1 = require("@angular-devkit/schematics");
const _utils_1 = require("@angular-toolkit/_utils");
const test_1 = require("@schematics/angular/utility/test");
const tasks_1 = require("@angular-devkit/schematics/tasks");
function addServerless(options) {
    options.serverless = {
        aws: {},
        gcloud: {}
    };
    const templateSource = schematics_1.apply(schematics_1.url('files/common'), [
        schematics_1.move(options.directory),
    ]);
    const rules = [];
    rules.push(schematics_1.mergeWith(templateSource, schematics_1.MergeStrategy.Overwrite));
    rules.push(_utils_1.addOrReplaceScriptInPackageJson(options, "deploy", "serverless deploy"));
    rules.push(_utils_1.addOrReplaceScriptInPackageJson(options, "build:deploy", "npm run build:prod && npm run deploy"));
    rules.push(_utils_1.addDependencyToPackageJson(options, 'serverless', '1.26.1', true));
    rules.push(_utils_1.addDependencyToPackageJson(options, 'ts-loader', '4.2.0', true));
    rules.push(_utils_1.addDependencyToPackageJson(options, 'webpack-cli', '2.1.2', true));
    if (options.provider === 'gcloud') {
        rules.push(addServerlessGcloud(options));
    }
    else if (options.provider === 'aws') {
        rules.push(addServerlessAWS(options));
    }
    else {
        options.serverless.aws.filename = 'serverless-aws.yml';
        options.serverless.gcloud.filename = 'serverless-gcloud.yml';
        rules.push(addServerlessAWS(options));
        rules.push(addServerlessGcloud(options));
        rules.push((tree, context) => {
            //add scripts to package.json
            const packageJsonSource = JSON.parse(test_1.getFileContent(tree, `${options.directory}/package.json`));
            delete packageJsonSource.scripts['build:deploy'];
            packageJsonSource.scripts['build:deploy:aws'] = 'npm run build:prod && npm run deploy:aws';
            packageJsonSource.scripts['build:deploy:gcloud'] = 'npm run build:prod && npm run deploy:gcloud';
            packageJsonSource.scripts['deploy:aws'] = 'cp-cli serverless-aws.yml serverless.yml && npm run deploy';
            packageJsonSource.scripts['deploy:gcloud'] = 'cp-cli serverless-gcloud.yml serverless.yml && npm run deploy';
            tree.overwrite(`${options.directory}/package.json`, JSON.stringify(packageJsonSource, null, "  "));
            if (!options.skipInstall) {
                context.addTask(new tasks_1.NodePackageInstallTask(options.directory));
            }
            return tree;
        });
    }
    rules.push((tree) => {
        const packageJsonSource = JSON.parse(test_1.getFileContent(tree, `${options.directory}/package.json`));
        const universal = isUniversal(tree, options);
        if (universal) {
            packageJsonSource.scripts['build:client-and-server-bundles'] = 'ng build --prod && ng run application:server';
            packageJsonSource.scripts['build:prod'] = 'npm run build:client-and-server-bundles && webpack --config webpack.server.config.js --progress --colors';
            tree.rename(`${options.directory}/server_universal.ts`, `${options.directory}/server.ts`);
            tree.rename(`${options.directory}/server_static.ts`, `${options.directory}temp/toRemove`);
        }
        else {
            packageJsonSource.scripts['build:prod'] = 'ng build --prod && webpack --config webpack.server.config.js --progress --colors';
            rules.push(_utils_1.addOrReplaceScriptInPackageJson(options, "build:prod", "ng build --prod && webpack --config webpack.server.config.js --progress --colors"));
            tree.rename(`${options.directory}/server_universal.ts`, `${options.directory}temp/toRemove`);
            tree.rename(`${options.directory}/server_static.ts`, `${options.directory}/server.ts`);
        }
        tree.overwrite(`${options.directory}/package.json`, JSON.stringify(packageJsonSource, null, "  "));
        const serverFileContent = test_1.getFileContent(tree, `${options.directory}/server.ts`);
        tree.overwrite(`${options.directory}/server.ts`, serverFileContent
            .replace('__distBrowserFolder__', getBrowserDistFolder(tree, options))
            .replace('__distServerFolder__', getServerDistFolder(tree, options)));
        return tree;
    });
    return schematics_1.chain(rules);
}
exports.default = addServerless;
function addServerlessAWS(options) {
    const fileName = options.serverless.aws.filename || 'serverless.yml';
    const source = schematics_1.apply(schematics_1.url('./files/aws'), [
        schematics_1.move(options.directory)
    ]);
    return schematics_1.chain([
        schematics_1.mergeWith(source),
        tree => {
            tree.rename(`${options.directory}/serverless-aws.yml`, `${options.directory}/${fileName}`);
            tree.overwrite(`${options.directory}/${fileName}`, test_1.getFileContent(tree, `${options.directory}/${fileName}`).replace('__appName__', options.project));
            return tree;
        },
        _utils_1.addDependencyToPackageJson(options, 'aws-serverless-express', '^3.2.0'),
        _utils_1.addDependencyToPackageJson(options, 'serverless-apigw-binary', '^0.4.4', true)
    ]);
}
function addServerlessGcloud(options) {
    const fileName = options.serverless.gcloud.filename || 'serverless.yml';
    const source = schematics_1.apply(schematics_1.url('./files/gcloud'), [
        schematics_1.move(options.directory)
    ]);
    return schematics_1.chain([
        schematics_1.mergeWith(source),
        tree => {
            tree.rename(`${options.directory}/serverless-gcloud.yml`, `${options.directory}/${fileName}`);
            tree.overwrite(`${options.directory}/${fileName}`, test_1.getFileContent(tree, `${options.directory}/${fileName}`).replace('__appName__', options.project));
            return tree;
        },
        _utils_1.addDependencyToPackageJson(options, 'firebase-admin', '^5.11.0'),
        _utils_1.addDependencyToPackageJson(options, 'firebase-functions', '^0.9.1'),
        _utils_1.addDependencyToPackageJson(options, 'serverless-google-cloudfunctions', '^1.1.1', true)
    ]);
}
function isUniversal(tree, options) {
    const cliConfig = JSON.parse(test_1.getFileContent(tree, `${options.directory}/angular.json`));
    const project = cliConfig.projects[options.project].architect;
    for (let property in project) {
        if (project.hasOwnProperty(property) && project[property].builder === '@angular-devkit/build-angular:server') {
            return true;
        }
    }
    return false;
}
function getServerDistFolder(tree, options) {
    const cliConfig = JSON.parse(test_1.getFileContent(tree, `${options.directory}/angular.json`));
    const project = cliConfig.projects[options.project].architect;
    for (let property in project) {
        if (project.hasOwnProperty(property) && project[property].builder === '@angular-devkit/build-angular:server') {
            return project[property].options.outputPath;
        }
    }
    return '';
}
function getBrowserDistFolder(tree, options) {
    const cliConfig = JSON.parse(test_1.getFileContent(tree, `${options.directory}/angular.json`));
    const project = cliConfig.projects[options.project].architect;
    for (let property in project) {
        if (project.hasOwnProperty(property) && project[property].builder === '@angular-devkit/build-angular:browser') {
            return project[property].options.outputPath;
        }
    }
    throw new schematics_1.SchematicsException('browser nor server builder not found!');
}
//# sourceMappingURL=index.js.map